package com.jiuqi.util;

package .tmp;

import com.jiuqi.util.StreamException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

public abstract class Stream {
  private boolean encode = true;
  
  private String charset = "GBK";
  
  private static final int BUFF_SIZE = 1024;
  
  public static final int SEEK_ORIGIN_START = 0;
  
  public static final int SEEK_ORIGIN_CURRENT = 1;
  
  public static final int SEEK_ORIGIN_END = 2;
  
  public void setUseEncode(boolean useEncode) {
    this.encode = useEncode;
  }
  
  public boolean getUseEncode() {
    return this.encode;
  }
  
  public void setCharset(String encodeCharset) {
    if (encodeCharset != null && encodeCharset.length() != 0)
      this.charset = encodeCharset; 
  }
  
  public String getCharset() {
    return this.charset;
  }
  
  public abstract void setSize(long paramLong) throws StreamException;
  
  public abstract int read(byte[] paramArrayOfbyte, int paramInt1, int paramInt2) throws StreamException;
  
  public abstract byte read() throws StreamException;
  
  public abstract int write(byte[] paramArrayOfbyte, int paramInt1, int paramInt2) throws StreamException;
  
  public abstract void write(byte paramByte) throws StreamException;
  
  public abstract long seek(long paramLong, int paramInt) throws StreamException;
  
  public void readBuffer(byte[] buffer, int pos, int count) throws StreamException {
    if (count != 0 && read(buffer, pos, count) != count)
      throw new StreamException("Stream read error"); 
  }
  
  public void writeBuffer(byte[] buffer, int pos, int count) throws StreamException {
    if (count != 0 && write(buffer, pos, count) != count)
      throw new StreamException("Stream write error"); 
  }
  
  public long copyFrom(com.jiuqi.util.Stream source, long count) throws StreamException {
    int bufSize;
    if (source == null)
      return 0L; 
    if (count == 0L) {
      source.setPosition(0L);
      count = source.getSize();
    } 
    if (count > 1024L) {
      bufSize = 1024;
    } else {
      bufSize = (int)count;
    } 
    byte[] buffer = new byte[1024];
    while (count != 0L) {
      int n;
      if (count > bufSize) {
        n = bufSize;
      } else {
        n = (int)count;
      } 
      source.readBuffer(buffer, 0, n);
      writeBuffer(buffer, 0, n);
      count -= n;
    } 
    return 0L;
  }
  
  public long copyFrom(InputStream stream, long size) throws StreamException, IOException {
    if (stream == null)
      return 0L; 
    long readcount = 0L;
    if (size == 0L) {
      byte[] buf = new byte[1024];
      int b = 0;
      do {
        b = stream.read(buf);
        if (b <= 0)
          continue; 
        write(buf, 0, b);
        readcount += b;
      } while (b > 0);
      while (b > 0) {
        b = stream.read(buf);
        if (b > 0)
          write(buf, 0, b); 
      } 
    } else {
      int readSize;
      byte[] buffer = new byte[1024];
      do {
        int blockSize = (size > 1024L) ? 1024 : (int)size;
        readSize = stream.read(buffer, 0, blockSize);
        write(buffer, 0, readSize);
        readcount += readSize;
        size -= readSize;
      } while (readSize > 0 && size > 0L);
    } 
    return readcount;
  }
  
  public long copyTo(OutputStream stream, long size) throws StreamException, IOException {
    long writecount = 0L;
    byte[] buffer = new byte[1024];
    int writesize = read(buffer, 0, 1024);
    while (writesize > 0 && size > 0L) {
      stream.write(buffer, 0, writesize);
      writesize = read(buffer, 0, 1024);
      writecount += writesize;
      size -= writesize;
    } 
    return writecount;
  }
  
  public byte[] getBytes() throws StreamException {
    long oldPos = getPosition();
    try {
      byte[] rt = new byte[(int)getSize()];
      setPosition(0L);
      read(rt, 0, rt.length);
      return rt;
    } finally {
      setPosition(oldPos);
    } 
  }
  
  public long getSize() throws StreamException {
    long pos = seek(0L, 1);
    long r = seek(0L, 2);
    seek(pos, 0);
    return r;
  }
  
  public long getPosition() throws StreamException {
    return seek(0L, 1);
  }
  
  public void setPosition(long pos) throws StreamException {
    seek(pos, 0);
  }
  
  public short readShort() throws StreamException {
    int b1 = read() & 0xFF;
    int b2 = read() & 0xFF;
    return (short)((b2 << 8) + (b1 << 0));
  }
  
  public int readWord() throws StreamException {
    int b1 = read() & 0xFF;
    int b2 = read() & 0xFF;
    return (b2 << 8) + (b1 << 0);
  }
  
  public int readInt() throws StreamException {
    int b1 = read() & 0xFF;
    int b2 = read() & 0xFF;
    int b3 = read() & 0xFF;
    int b4 = read() & 0xFF;
    return (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
  }
  
  public float readFloat() throws StreamException {
    return Float.intBitsToFloat(readInt());
  }
  
  public double readDouble() throws StreamException {
    return Double.longBitsToDouble(readLong());
  }
  
  public String readString(int length) throws StreamException {
    byte[] str = new byte[length];
    read(str, 0, length);
    return decodeString(str);
  }
  
  public String decodeString(byte[] data) {
    if (this.encode)
      try {
        return new String(data, this.charset);
      } catch (UnsupportedEncodingException unsupportedEncodingException) {
        return new String(data);
      }  
    return new String(data);
  }
  
  public String readStringBySize() throws StreamException {
    int len = readInt();
    if (len == 0)
      return null; 
    return readString(len);
  }
  
  public boolean readBool() throws StreamException {
    return (read() > 0);
  }
  
  public long readLong() throws StreamException {
    long b1 = (read() & 0xFF);
    long b2 = (read() & 0xFF);
    long b3 = (read() & 0xFF);
    long b4 = (read() & 0xFF);
    long b5 = (read() & 0xFF);
    long b6 = (read() & 0xFF);
    long b7 = (read() & 0xFF);
    long b8 = read();
    return (b8 << 56L) + (b7 << 48L) + (b6 << 40L) + (b5 << 32L) + (b4 << 24L) + (b3 << 16L) + (b2 << 8L) + (b1 << 0L);
  }
  
  public void writeShort(short value) throws StreamException {
    write((byte)(value >>> 0 & 0xFF));
    write((byte)(value >>> 8 & 0xFF));
  }
  
  public void writeWord(int value) throws StreamException {
    write((byte)(value >>> 0 & 0xFF));
    write((byte)(value >>> 8 & 0xFF));
  }
  
  public void writeInt(int value) throws StreamException {
    write((byte)(value >>> 0 & 0xFF));
    write((byte)(value >>> 8 & 0xFF));
    write((byte)(value >>> 16 & 0xFF));
    write((byte)(value >>> 24 & 0xFF));
  }
  
  public void writeLong(long value) throws StreamException {
    write((byte)(int)(value >>> 0L & 0xFFL));
    write((byte)(int)(value >>> 8L & 0xFFL));
    write((byte)(int)(value >>> 16L & 0xFFL));
    write((byte)(int)(value >>> 24L & 0xFFL));
    write((byte)(int)(value >>> 32L & 0xFFL));
    write((byte)(int)(value >>> 40L & 0xFFL));
    write((byte)(int)(value >>> 48L & 0xFFL));
    write((byte)(int)(value >>> 56L & 0xFFL));
  }
  
  public void writeFloat(float value) throws StreamException {
    writeInt(Float.floatToIntBits(value));
  }
  
  public void writeDouble(double value) throws StreamException {
    writeLong(Double.doubleToLongBits(value));
  }
  
  public void writeBool(boolean value) throws StreamException {
    byte b = value ? 1 : 0;
    write(b);
  }
  
  public void writeString(String value) throws StreamException {
    byte[] b = encodeString(value);
    write(b, 0, b.length);
  }
  
  public byte[] encodeString(String value) {
    if (value == null)
      return null; 
    if (this.encode)
      try {
        return value.getBytes(this.charset);
      } catch (UnsupportedEncodingException unsupportedEncodingException) {
        return value.getBytes();
      }  
    return value.getBytes();
  }
  
  public void writeStringWithSize(String value) throws StreamException {
    if (value == null) {
      writeInt(0);
      return;
    } 
    byte[] b = encodeString(value);
    writeInt(b.length);
    write(b, 0, b.length);
  }
  
  public void loadFromFile(String fileName) throws StreamException, IOException {
    InputStream source = new FileInputStream(fileName);
    try {
      loadFromStream(source);
    } finally {
      source.close();
    } 
  }
  
  public void loadFromStream(InputStream inStream) throws StreamException, IOException {
    copyFrom(inStream, 0L);
  }
  
  public void saveToFile(String fileName) throws StreamException, IOException {
    OutputStream dest = new FileOutputStream(fileName);
    try {
      saveToStream(dest);
    } finally {
      dest.close();
    } 
  }
  
  public void saveToStream(OutputStream outStream) throws StreamException, IOException {
    setPosition(0L);
    copyTo(outStream, getSize());
  }
}
